%eyam
% GUI for Eyam S,I,D epidemic model.
%
% LAST UPDATED by Andy French Jun 2019

function varargout = eyam(varargin)

%% Initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @eyam_OpeningFcn, ...
    'gui_OutputFcn',  @eyam_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

%% Harmonograph opening function. This executes just before eyam is made visible.
function eyam_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for eyam
handles.output = hObject;

%Turn off warnings (e.g. 'divide by zero')
warning off

%Set up global variable d which can be accessed by all functions
global d
d.h = handles;

%Center figure
figcentre( d.h.FIGUREeyam );

%Set defaults
eyam_defaults

%Update eyam
eyam_update;

%%

% Default parameters
function eyam_defaults
global d

% Max I0, S0 values etc
d.I0max = 500;
d.S0max = 1000;
d.amax = 50;
d.aoverbmax = 500;
d.dtmax = 0.5;

% Eyam model alpha and beta parameters
d.a = 2.9869;
d.aoverb = 163.2884;

%Define timestep
d.dt = 0.005;

%Define maximum time t value /months
d.tmax = 4;

%Fontsize and markersize for plots
d.fsize = 14;
d.msize = 14;
d.plot_and_save_PNG = 0;

% Mompesson Eyam data
d.t_mompesson = [0,0.51,1.02,1.53,2.04,2.55,3.57];
d.I_mompesson = [14.5,22,29,21,8,8,0];
d.S_mompesson = [235,201,153.5,121,108,97,83];
d.I0 = d.I_mompesson(1);
d.S0 = d.S_mompesson(1);
d.D_mompesson = d.I0+d.S0 - d.I_mompesson - d.S_mompesson;
d.overlay_mompesson = 1;

%Set GUI defaults
set( d.h.EDITtmax,'string', num2str(d.tmax) );
set( d.h.EDITdt,'string', num2str(d.dt) );
set( d.h.EDITI0,'string', num2str(d.I0) );
set( d.h.EDITS0,'string', num2str(d.S0) );
set( d.h.EDITa,'string', num2str(d.a,4) );
set( d.h.EDITaoverb,'string', num2str(d.aoverb,4) );
set( d.h.SLIDERI0,'value',d.I0/d.I0max );
set( d.h.SLIDERS0,'value',d.S0/d.S0max );
set( d.h.SLIDERa,'value',d.a/d.amax );
set( d.h.SLIDERaoverb,'value',d.aoverb/d.aoverbmax );
set( d.h.SLIDERdt,'value',d.dt/d.dtmax );
set( d.h.TOGGLEoverlaymompesson,'value',1 );

%%

%% eyam output function. Outputs from this function are returned to the command line.
function varargout = eyam_OutputFcn(hObject, eventdata, handles)
global d

% Get default command line output from handles structure
varargout{1} = d.h.output;

%%

%% Save PNG of graph pushbutton
function PUSHsavepng_Callback(hObject, eventdata, handles)
global d
d.plot_and_save_PNG = 1;
eyam_update;
d.plot_and_save_PNG = 0;

%%

%% Overlay Mompesson data togglebutton
function TOGGLEoverlaymompesson_Callback(hObject, eventdata, handles)
global d
if get(d.h.TOGGLEoverlaymompesson,'value')==1
    d.overlay_mompesson = 1;
else
    d.overlay_mompesson = 0;
end;
eyam_update;

%%

%% I0 slider
function SLIDERI0_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERI0,'value');

%Determine vale of I0 and update associated edit box
d.I0 = v * d.I0max;
set(d.h.EDITI0,'string',num2str(d.I0,4));

%Update GUI
eyam_update

%%

%% S0 slider
function SLIDERS0_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERS0,'value');

%Determine vale of S0 and update associated edit box
d.S0 = v * d.S0max;
set(d.h.EDITS0,'string',num2str(d.S0,4));

%Update GUI
eyam_update

%%

%% alpha slider
function SLIDERa_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERa,'value');

%Determine vale of a and update associated edit box
d.a = v * d.amax;
set(d.h.EDITa,'string',num2str(d.a,4));

%Update GUI
eyam_update

%%

%% alpha/beta slider
function SLIDERaoverb_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERaoverb,'value');

%Determine vale of a/b and update associated edit box
d.aoverb = v*d.aoverbmax;
set(d.h.EDITaoverb,'string',num2str(d.aoverb,4));

%Update GUI
eyam_update

%%

%% dt slider
function SLIDERdt_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERdt,'value');

%Determine vale of a/b and update associated edit box
d.dt = v*d.dtmax;
set(d.h.EDITdt,'string',num2str(d.dt,5));

%Update GUI
eyam_update

%%

%% I0 edit box
function EDITI0_Callback(hObject, eventdata, handles)
global d

%Get numerical value of I0
d.I0 = str2num( get(handles.EDITI0,'string') );

%Check limits and clip if necessary
if d.I0 > d.I0max
    d.I0 = d.I0max;
elseif d.I0 < 0
    d.I0 = 0;
end

%Update EDIT box
set(handles.EDITI0,'string',num2str( d.I0 ) );

%Update slider
set(handles.SLIDERI0,'value',d.I0/d.I0max );

%Update model
eyam_update

%%

%% S0 edit box
function EDITS0_Callback(hObject, eventdata, handles)
global d

%Get numerical value of S0
d.S0 = str2num( get(handles.EDITS0,'string') );

%Check limits and clip if necessary
if d.S0 > d.S0max
    d.S0 = d.S0max;
elseif d.S0 < 0
    d.S0 = 0;
end

%Update EDIT box
set(handles.EDITS0,'string',num2str( d.S0 ) );

%Update slider
set(handles.SLIDERS0,'value',d.S0/d.S0max );

%Update model
eyam_update

%%

%% alpha edit box
function EDITa_Callback(hObject, eventdata, handles)
global d

%Get numerical value of alpha
d.a = str2num( get(handles.EDITa,'string') );

%Check limits and clip if necessary
if d.a > d.amax
    d.a = d.amax;
elseif d.a < 0
    d.a = 0;
end

%Update EDIT box
set(handles.EDITa,'string',num2str( d.a ) );

%Update slider
set(handles.SLIDERa,'value',d.a/d.amax );

%Update model
eyam_update

%%

%% alpha/beta edit box
function EDITaoverb_Callback(hObject, eventdata, handles)
global d

%Get numerical value of alpha/beta
d.aoverb = str2num( get(handles.EDITaoverb,'string') );

%Check limits and clip if necessary
if d.aoverb > d.aoverbmax
    d.aoverb = d.aoverbmax;
elseif d.aoverb < 0
    d.aoverb = 0;
end

%Update EDIT box
set(handles.EDITaoverb,'string',num2str( d.aoverb ) );

%Update slider
set(handles.SLIDERI0,'value',d.aoverb/d.aoverbmax );

%Update model
eyam_update

%%

%% dt edit box
function EDITdt_Callback(hObject, eventdata, handles)
global d

%Get numerical value of dt
d.dt = str2num( get(handles.EDITdt,'string') );

%Check limits and clip if necessary
if d.dt > d.dtmax
    d.dt = d.dtmax;
elseif d.dt < 0.001
    d.dt = 0.001;
end

%Update EDIT box
set(handles.EDITdt,'string',num2str( d.dt ) );

%Update slider
set(handles.SLIDERdt,'value',d.dt/d.dtmax );

%Update model
eyam_update

%%

%% tmaxedit box
function EDITtmax_Callback(hObject, eventdata, handles)
global d

%Get numerical value of tmax
d.tmax = str2num( get(handles.EDITtmax,'string') );

%Update model
eyam_update

%%

%Update GUI
function eyam_update
global d

%Update Eyam model
d.b = d.a/d.aoverb;
[t,I,S,D,Imax,Smin,Dmax,tImax] = eyam_model( d.dt, d.I0, d.S0, d.a, d.b, d.tmax );
set( d.h.TEXTImaxSminDmax,'string',{['Imax = ',num2str(Imax,4),', tImax = ',num2str(tImax,4)],...
    ['Smin = ',num2str(Smin,4)],['Dmax = ',num2str(Dmax,4)]} );

if d.plot_and_save_PNG==1
    %Create new figure
    fig = figure;
    figure(fig);
    fsize = 22;
else
    %Point to axes
    figure( d.h.FIGUREeyam );
    axes( d.h.AXESeyam );
    fsize = d.fsize;
    cla;
end

%Plot Eyam model and overlay Mompesson data
plot( t,I,'r-','linewidth',2); hold on;
plot( t,S,'g-','linewidth',2);
plot( t,D,'b-','linewidth',2);
if d.overlay_mompesson==1
    plot( d.t_mompesson, d.I_mompesson,'r+', 'markersize', d.msize );
    plot( d.t_mompesson, d.S_mompesson,'g+', 'markersize', d.msize );
    plot( d.t_mompesson, d.D_mompesson,'b+', 'markersize', d.msize );
end
xlabel( 't /months', 'fontsize', fsize );
ylabel( 'Eyam population', 'fontsize', fsize );
title( ['Eyam Euler model: \alpha = ',num2str( d.a,3),...
    ', \beta = ',num2str(d.b,3),', \Deltat = ',num2str(d.dt)], 'fontsize', fsize )
set( gca,'fontsize', fsize );
legend( {'I','S','D'}, 'fontsize', fsize );
xlim( [0, d.tmax]  ); ylim( [0, d.S0 + d.I0] )
grid on

%Save externally plotted graph, if option set
if d.plot_and_save_PNG==1
    png_filename = [ 'eyam alpha=',strrep(num2str(d.a,4),'.','p'),' beta=',...
        strrep(num2str(d.b,3),'.','p'),' dt=',strrep(num2str(d.dt,5),'.','p'),...
        ' I0=',strrep(num2str(d.I0,3),'.','p'),' S0=',strrep(num2str(d.S0,3),'.','p'),'.png'];
    print( fig,png_filename,'-dpng','-r300');
    close(fig);
    figure(d.h.FIGUREeyam); axes(d.h.AXESeyam);
end

%%

%Euler method solver for differential equations which
%describe model of Eyam epidemic.
function [t,I,S,D,Imax,Smin,Dmax,tImax] = eyam_model( dt, I0, S0, alpha, beta, tmax )

%Initialize output vectors for t,I,S,D
t = 0 : dt : tmax;
N = length(t);
S = S0*ones(1,N);
I = I0*ones(1,N);
D = zeros(1,N);

%Loop through vectors to compute t, I, S, D.
%using the Euler first order differential equation method
for n=2:N
    t(n) = t(n-1) + dt;
    I(n) = I(n-1) + dt*( beta*S(n-1)*I(n-1) - alpha*I(n-1) );
    S(n) = S(n-1) - dt*beta*S(n-1)*I(n-1);
    D(n) = D(n-1) + dt*alpha*I(n-1);
end

%Find Imax, Smin, Dmax
Imax = I0 + S0 - (alpha/beta)*( 1 + log(S0*beta/alpha) );

%Find tmax
i = find( I==max(I) );
tImax = t(i(1));

% Determine minimum S (when I=0) via iterative method
s = [ 0, S0/exp( (beta/alpha)*(I0+S0)) ];
y = [ I0+S0, (alpha/beta)*log(S0/s(end)) ];
d = abs( (alpha/beta)*log(S0/s(end)) - (I0+S0 - s(end)) );
while (d > 1e-6)
    y = [ y,I0 + S0 - s(end) ];
    s = [ s,s(end) ];   
    d = abs( (alpha/beta)*log(S0/s(end)) - (I0+S0 - s(end)) );
    s = [ s,S0/exp( (beta/alpha)*y(end) )];
    y = [ y,(alpha/beta)*log(S0/s(end)) ];
end
Smin = s(end);
Dmax = (alpha/beta)*log(S0/Smin);

%%

%figcenter
% Centres a figure with handle fig in the current display.
function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*screen_size(3) - 0.5*fig_position(3);
centre_y = 0.5*screen_size(4) - 0.5*fig_position(4);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%End of code






