%eyam_probability_map
% Generates probability vs time coloured surfaces for S,I,D for Eyam data 
% generated by the stochastic model.
%
% LAST UPDATED by Andy French June 2019

function eyam_probability_map

%Set colour range (log of probability)
cmax = 0; cmin = -10;

%Load data
load eyam_stochastic

%Determine size of S,I,D matrices
[N,M] = size(SSs);

%Define 'bins' for S,I,D
Sbins = 0 : ceil(I0 + S0);
Ibins = 0 : ceil(I0 + S0);
Dbins = 0 : ceil(I0 + S0);

%Compute matrix of probabilities for each timestep
PS = zeros( length(Sbins), N );
PI = zeros( length(Ibins), N );
PD = zeros( length(Dbins), N );
for n=1:N
    PS(:,n) = prob_vs_t( SSs(n,:), Sbins ).';
    PI(:,n) = prob_vs_t( IIs(n,:), Ibins ).';
    PD(:,n) = prob_vs_t( DDs(n,:), Dbins ).';
end

%Plot probability surface

%S
pcolor(t,Sbins,log(PS)); hold on;
interp_colormap(1000);
shading interp;
caxis([cmin,cmax]);
xlabel('t /months', 'fontsize', fsize );
ylabel('Susceptibles (S)', 'fontsize', fsize );
set(gca, 'fontsize', fsize );
colorbar('fontsize', fsize );
plot( ts,Ss,'g-','linewidth',2);
plot( t,S,'k--','linewidth',2);
plot( t_data, S_data,'ko', 'markersize', msize );  
title( ['log( P(S,t) ) : \alpha=',num2str(a,3),...
        ', \beta=',num2str(b,3),', \Deltat=',num2str(dt)], 'fontsize', fsize )
print( gcf, 'S probability vs t.png','-dpng','-r300' );
clf;

%I
pcolor(t,Ibins,log(PI)); hold on
interp_colormap(1000);
shading interp;
caxis([cmin,cmax]);
xlabel('t /months', 'fontsize', fsize );
ylabel('Infectives (I)', 'fontsize', fsize );
set(gca, 'fontsize', fsize );
colorbar('fontsize', fsize );
plot( ts,Is,'r-','linewidth',2);
plot( t,I,'k--','linewidth',2);
plot( t_data, I_data,'ko', 'markersize', msize );
title( ['log( P(I,t) ) : \alpha=',num2str(a,3),...
        ', \beta=',num2str(b,3),', \Deltat=',num2str(dt)], 'fontsize', fsize )
print( gcf, 'I probability vs t.png','-dpng','-r300' );
clf;

%D
pcolor(t,Dbins,log(PD)); hold on
interp_colormap(1000);
shading interp;
caxis([cmin,cmax]);
xlabel('t /months', 'fontsize', fsize );
ylabel('Dead (D)', 'fontsize', fsize );
set(gca, 'fontsize', fsize );
colorbar('fontsize', fsize );
plot( ts,Ds,'b-','linewidth',2);
plot( t,D,'k--','linewidth',2);
plot( t_data, D_data,'ko', 'markersize', msize );
title( ['log( P(D,t) ) : \alpha=',num2str(a,3),...
        ', \beta=',num2str(b,3),', \Deltat=',num2str(dt)], 'fontsize', fsize )
print( gcf, 'D probability vs t.png','-dpng','-r300' );
close(gcf);

%%

%Compute probability of sample values xbins based upon sample vector x
function p = prob_vs_t(x,xbins)
p = histc( x, xbins );
p = p./sum(p);

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap( N )

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code